@@ -5,11 +5,13 @@ import math
 import numpy as np
 from datetime import datetime, timedelta
 import rsudp.raspberryshake as rs
-from rsudp import printM, printW, printE
+from rsudp import printM, printW, printE, helpers
 import rsudp
 import linecache
 sender = 'plot.py'
 QT = False
+QtGui = False
+PhotoImage = False
 try:		# test for matplotlib and exit if import fails
 	from matplotlib import use
 	try:	# no way to know what machines can handle what software, but Tk is more universal
@@ -45,8 +47,8 @@ except Exception as e:
 	printE('detail: %s' % e, sender, spaces=True)
 	MPL = False
 
-icon = 'icon.ico'
-icon2 = 'icon.png'
+ICON = 'icon.ico'
+ICON2 = 'icon.png'
 
 class Plot:
 	'''
@@ -76,10 +78,36 @@ class Plot:
 	:param bool alert: whether to draw the number of events at startup. Defaults to True.
 	:param queue.Queue q: queue of data and messages sent by :class:`rsudp.c_consumer.Consumer`
 	:raise ImportError: if the module cannot import either of the Matplotlib Qt5 or TkAgg backends
-
 	'''
+	def _set_deconv(self, deconv):
+		'''
+		This function sets the deconvolution units. Allowed values are as follows:
 
-	def __init__(self, cha='all', q=False,
+		.. |ms2| replace:: m/s\ :sup:`2`\
+
+		- ``'VEL'`` - velocity (m/s)
+		- ``'ACC'`` - acceleration (|ms2|)
+		- ``'GRAV'`` - fraction of acceleration due to gravity (g, or 9.81 |ms2|)
+		- ``'DISP'`` - displacement (m)
+		- ``'CHAN'`` - channel-specific unit calculation, i.e. ``'VEL'`` for geophone channels and ``'ACC'`` for accelerometer channels
+
+		:param str deconv: ``'VEL'``, ``'ACC'``, ``'GRAV'``, ``'DISP'``, or ``'CHAN'``
+		'''
+		self.deconv = deconv if (deconv in rs.UNITS) else False
+		if self.deconv and rs.inv:
+			deconv = deconv.upper()
+			if self.deconv in rs.UNITS:
+				self.units = rs.UNITS[self.deconv][0]
+				self.unit = rs.UNITS[self.deconv][1]
+			printM('Signal deconvolution set to %s' % (self.deconv), self.sender)
+		else:
+			self.units = rs.UNITS['CHAN'][0]
+			self.unit = rs.UNITS['CHAN'][1]
+			self.deconv = False
+		printM('Seismogram units are %s' % (self.units), self.sender)
+
+
+	def __init__(self, q, cha='all',
 				 seconds=30, spectrogram=True,
 				 fullscreen=False, kiosk=False,
 				 deconv=False, screencap=False,
@@ -99,49 +127,25 @@ class Plot:
 			sys.exit()
 		if QT == False:
 			printW('Running on %s machine, using Tk instead of Qt' % (platform.machine()), self.sender)
-		if q:
-			self.queue = q
-		else:
-			printE('no queue passed to consumer! Thread will exit now!', self.sender)
-			sys.stdout.flush()
-			sys.exit()
 
+		self.queue = q
 		self.master_queue = None	# careful with this, this goes directly to the master consumer. gets set by main thread.
 
 		self.stream = rs.Stream()
 		self.raw = rs.Stream()
 		self.stn = rs.stn
 		self.net = rs.net
+
 		self.chans = []
-		cha = rs.chns if (cha == 'all') else cha
-		cha = list(cha) if isinstance(cha, str) else cha
-		l = rs.chns
-		for c in l:
-			n = 0
-			for uch in cha:
-				if (uch.upper() in c) and (c not in str(self.chans)):
-					self.chans.append(c)
-				n += 1
-		if len(self.chans) < 1:
-			self.chans = rs.chns
-		printM('Plotting channels: %s' % self.chans, self.sender)
+		helpers.set_channels(self, cha)
+		printM('Plotting %s channels: %s' % (len(self.chans), self.chans), self.sender)
 		self.totchns = rs.numchns
+
 		self.seconds = seconds
 		self.pkts_in_period = rs.tr * rs.numchns * self.seconds	# theoretical number of packets received in self.seconds
 		self.spectrogram = spectrogram
 
-		self.deconv = deconv if (deconv in rs.UNITS) else False
-		if self.deconv and rs.inv:
-			deconv = deconv.upper()
-			if self.deconv in rs.UNITS:
-				self.units = rs.UNITS[self.deconv][0]
-				self.unit = rs.UNITS[self.deconv][1]
-			printM('Signal deconvolution set to %s' % (self.deconv), self.sender)
-		else:
-			self.units = rs.UNITS['CHAN'][0]
-			self.unit = rs.UNITS['CHAN'][1]
-			self.deconv = False
-		printM('Seismogram units are %s' % (self.units), self.sender)
+		self._set_deconv(deconv)
 
 		self.per_lap = 0.9
 		self.fullscreen = fullscreen
@@ -169,7 +173,7 @@ class Plot:
 		'''
 		Send the streams to the central library deconvolve function.
 		'''
-		rs.deconvolve(self)
+		helpers.deconvolve(self)
 
 	def getq(self):
 		'''
@@ -192,7 +196,7 @@ class Plot:
 			self.events += 1		# add event to count
 			self.save_timer -= 1	# don't push the save time forward if there are a large number of alarm events
 			event = [self.save_timer + int(self.save_pct*self.pkts_in_period),
-					 rs.fsec(rs.get_msg_time(d))]	# event = [save after count, datetime]
+					 helpers.fsec(helpers.get_msg_time(d))]	# event = [save after count, datetime]
 			self.last_event_str = '%s UTC' % (event[1].strftime('%Y-%m-%d %H:%M:%S.%f')[:22])
 			printM('Event time: %s' % (self.last_event_str), sender=self.sender)		# show event time in the logs
 			if self.screencap:
@@ -245,7 +249,7 @@ class Plot:
 		Handles a plot close event.
 		This will trigger a full shutdown of all other processes related to rsudp.
 		'''
-		self.master_queue.put(rs.msg_term())
+		self.master_queue.put(helpers.msg_term())
 
 	def handle_resize(self, evt=False):
 		'''
@@ -302,7 +306,8 @@ class Plot:
 		printM('Saved %s' % (figname), sender=self.sender)
 		printM('%s thread has saved an image, sending IMGPATH message to queues' % self.sender, sender=self.sender)
 		# imgpath requires a UTCDateTime and a string figure path
-		self.master_queue.put(rs.msg_imgpath(event_time, figname))
+		self.master_queue.put(helpers.msg_imgpath(event_time, figname))
+
 
 	def _set_fig_title(self):
 		'''
@@ -313,13 +318,10 @@ class Plot:
 						  fontsize=14, color=self.fgcolor, x=0.52)
 
 
-	def setup_plot(self):
-		"""
-		Sets up the plot. Quite a lot of stuff happens in this function.
-		Matplotlib backends are not threadsafe, so things are a little weird.
-		See code comments for details.
-		"""
-		# instantiate a figure and set basic params
+	def _init_plot(self):
+		'''
+		Initialize plot elements and calculate parameters.
+		'''
 		self.fig = plt.figure(figsize=(11,3*self.num_chans))
 		self.fig.canvas.mpl_connect('close_event', self.handle_close)
 		self.fig.canvas.mpl_connect('resize_event', self.handle_resize)
@@ -343,50 +345,48 @@ class Plot:
 			self.nfft1 = self._nearest_pow_2(self.sps)
 			self.nlap1 = self.nfft1 * self.per_lap
 
-		for i in range(self.num_chans):
-			if i == 0:
-				# set up first axes (axes added later will share these x axis limits)
+
+	def _init_axes(self, i):
+		'''
+		Initialize plot axes.
+		'''
+		if i == 0:
+			# set up first axes (axes added later will share these x axis limits)
+			self.ax.append(self.fig.add_subplot(self.num_chans*self.mult,
+							1, 1, label=str(1)))
+			self.ax[0].set_facecolor(self.bgcolor)
+			self.ax[0].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
+			self.ax[0].xaxis.set_major_formatter(mdates.DateFormatter('%H:%M:%S'))
+			self.ax[0].yaxis.set_major_formatter(EngFormatter(unit='%s' % self.unit.lower()))
+			if self.spectrogram:
 				self.ax.append(self.fig.add_subplot(self.num_chans*self.mult,
-							   1, 1, label=str(1)))
-				self.ax[0].set_facecolor(self.bgcolor)
-				self.ax[0].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
-				self.ax[0].xaxis.set_major_formatter(mdates.DateFormatter('%H:%M:%S'))
-				self.ax[0].yaxis.set_major_formatter(EngFormatter(unit='%s' % self.unit.lower()))
-				if self.spectrogram:
-					self.ax.append(self.fig.add_subplot(self.num_chans*self.mult,
-								   1, 2, label=str(2)))#, sharex=ax[0]))
-					self.ax[1].set_facecolor(self.bgcolor)
-					self.ax[1].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
-			else:
-				# add axes that share either lines or spectrogram axis limits
-				s = i * self.mult	# plot selector
-				# add a subplot then set colors
+								1, 2, label=str(2)))#, sharex=ax[0]))
+				self.ax[1].set_facecolor(self.bgcolor)
+				self.ax[1].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
+		else:
+			# add axes that share either lines or spectrogram axis limits
+			s = i * self.mult	# plot selector
+			# add a subplot then set colors
+			self.ax.append(self.fig.add_subplot(self.num_chans*self.mult,
+							1, s+1, sharex=self.ax[0], label=str(s+1)))
+			self.ax[s].set_facecolor(self.bgcolor)
+			self.ax[s].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
+			self.ax[s].xaxis.set_major_formatter(mdates.DateFormatter('%H:%M:%S'))
+			self.ax[s].yaxis.set_major_formatter(EngFormatter(unit='%s' % self.unit.lower()))
+			if self.spectrogram:
+				# add a spectrogram and set colors
 				self.ax.append(self.fig.add_subplot(self.num_chans*self.mult,
-							   1, s+1, sharex=self.ax[0], label=str(s+1)))
-				self.ax[s].set_facecolor(self.bgcolor)
-				self.ax[s].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
-				self.ax[s].xaxis.set_major_formatter(mdates.DateFormatter('%H:%M:%S'))
-				self.ax[s].yaxis.set_major_formatter(EngFormatter(unit='%s' % self.unit.lower()))
-				if self.spectrogram:
-					# add a spectrogram and set colors
-					self.ax.append(self.fig.add_subplot(self.num_chans*self.mult,
-								   1, s+2, sharex=self.ax[1], label=str(s+2)))
-					self.ax[s+1].set_facecolor(self.bgcolor)
-					self.ax[s+1].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
+								1, s+2, sharex=self.ax[1], label=str(s+2)))
+				self.ax[s+1].set_facecolor(self.bgcolor)
+				self.ax[s+1].tick_params(colors=self.fgcolor, labelcolor=self.fgcolor)
 
-		for axis in self.ax:
-			# set the rest of plot colors
-			plt.setp(axis.spines.values(), color=self.fgcolor)
-			plt.setp([axis.get_xticklines(), axis.get_yticklines()], color=self.fgcolor)
 
-		# calculate times
-		start = np.datetime64(self.stream[0].stats.endtime
-							  )-np.timedelta64(self.seconds, 's')	# numpy time
-		end = np.datetime64(self.stream[0].stats.endtime)	# numpy time
-
-		# rs logos
+	def _set_icon(self):
+		'''
+		Set RS plot icons.
+		'''
 		mgr = plt.get_current_fig_manager()
-		ico = pr.resource_filename('rsudp', os.path.join('img', icon))
+		ico = pr.resource_filename('rsudp', os.path.join('img', ICON))
 		if QT:
 			mgr.window.setWindowIcon(QtGui.QIcon(ico))
 		else:
@@ -396,12 +396,22 @@ class Plot:
 			except:
 				printW('Failed to set PNG icon image, trying .ico instead', sender=self.sender)
 				try:
-					ico = pr.resource_filename('rsudp', os.path.join('img', icon2))
+					ico = pr.resource_filename('rsudp', os.path.join('img', ICON2))
 					ico = PhotoImage(file=ico)
 					mgr.window.tk.call('wm', 'iconphoto', mgr.window._w, ico)
 				except:
 					printE('Failed to set window icon.')
 
+
+	def _format_axes(self):
+		'''
+		Setting up axes and artists.
+		'''
+		# calculate times
+		start = np.datetime64(self.stream[0].stats.endtime
+							  )-np.timedelta64(self.seconds, 's')	# numpy time
+		end = np.datetime64(self.stream[0].stats.endtime)	# numpy time
+
 		im = mpimg.imread(pr.resource_filename('rsudp', os.path.join('img', 'version1-01-small.png')))
 		self.imax = self.fig.add_axes([0.015, 0.944, 0.2, 0.056], anchor='NW') # [left, bottom, right, top]
 		self.imax.imshow(im, aspect='equal', interpolation='sinc')
@@ -442,7 +452,11 @@ class Plot:
 						extent=(self.seconds-(1/(self.sps/float(len(self.stream[i].data)))),
 								self.seconds,0,self.sps/2), aspect='auto')
 
-		self.handle_resize()
+
+	def _setup_fig_manager(self):
+		'''
+		Setting up figure manager and 
+		'''
 		# update canvas and draw
 		figManager = plt.get_current_fig_manager()
 		if self.kiosk:
@@ -455,11 +469,108 @@ class Plot:
 					figManager.resize(*figManager.window.maxsize())
 
 
+	def setup_plot(self):
+		"""
+		Sets up the plot. Quite a lot of stuff happens in this function.
+		Matplotlib backends are not threadsafe, so things are a little weird.
+		See code comments for details.
+		"""
+		# instantiate a figure and set basic params
+		self._init_plot()
+
+		for i in range(self.num_chans):
+			self._init_axes(i)
+
+		for axis in self.ax:
+			# set the rest of plot colors
+			plt.setp(axis.spines.values(), color=self.fgcolor)
+			plt.setp([axis.get_xticklines(), axis.get_yticklines()], color=self.fgcolor)
+
+		# rs logos
+		self._set_icon()
+
+		# draw axes
+		self._format_axes()
+
+		self.handle_resize()
+
+		# setup figure manager
+		self._setup_fig_manager()
+
+		# draw plot, loop, and resize the plot
 		plt.draw()									# draw the canvas
 		self.fig.canvas.start_event_loop(0.005)		# wait for canvas to update
 		self.handle_resize()
 
 
+	def _set_ch_specific_label(self, i):
+		'''
+		Set the formatter units if the deconvolution is channel-specific.
+		'''
+		if self.deconv:
+			if (self.deconv in 'CHAN'):
+				ch = self.stream[i].stats.channel
+				if ('HZ' in ch) or ('HN' in ch) or ('HE' in ch):
+					unit = rs.UNITS['VEL'][1]
+				elif ('EN' in ch):
+					unit = rs.UNITS['ACC'][1]
+				else:
+					unit = rs.UNITS['CHAN'][1]
+				self.ax[i*self.mult].yaxis.set_major_formatter(EngFormatter(unit='%s' % unit.lower()))
+
+
+	def _draw_lines(self, i, start, end, mean):
+		'''
+		Updates the line data in the plot.
+
+		:param int i: the trace number
+		:param numpy.datetime64 start: start time of the trace
+		:param numpy.datetime64 end: end time of the trace
+		:param float mean: the mean of data in the trace
+		'''
+		comp = 1/self.per_lap	# spectrogram offset compensation factor
+		r = np.arange(start, end, np.timedelta64(int(1000/self.sps), 'ms'))[-len(
+					self.stream[i].data[int(-self.sps*(self.seconds-(comp/2))):-int(self.sps*(comp/2))]):]
+		self.lines[i].set_ydata(self.stream[i].data[int(-self.sps*(self.seconds-(comp/2))):-int(self.sps*(comp/2))]-mean)
+		self.lines[i].set_xdata(r)	# (1/self.per_lap)/2
+		self.ax[i*self.mult].set_xlim(left=start.astype(datetime)+timedelta(seconds=comp*1.5),
+										right=end.astype(datetime))
+		self.ax[i*self.mult].set_ylim(bottom=np.min(self.stream[i].data-mean)
+										-np.ptp(self.stream[i].data-mean)*0.1,
+										top=np.max(self.stream[i].data-mean)
+										+np.ptp(self.stream[i].data-mean)*0.1)
+
+
+	def _update_specgram(self, i, mean):
+		'''
+		Updates the spectrogram and its labels.
+
+		:param int i: the trace number
+		:param float mean: the mean of data in the trace
+		'''
+		self.nfft1 = self._nearest_pow_2(self.sps)	# FFTs run much faster if the number of transforms is a power of 2
+		self.nlap1 = self.nfft1 * self.per_lap
+		if len(self.stream[i].data) < self.nfft1:	# when the number of data points is low, we just need to kind of fake it for a few fractions of a second
+			self.nfft1 = 8
+			self.nlap1 = 6
+		sg = self.ax[i*self.mult+1].specgram(self.stream[i].data-mean,
+					NFFT=self.nfft1, pad_to=int(self.nfft1*4), # previously self.sps*4),
+					Fs=self.sps, noverlap=self.nlap1)[0]	# meat & potatoes
+		self.ax[i*self.mult+1].clear()	# incredibly important, otherwise continues to draw over old images (gets exponentially slower)
+		# cloogy way to shift the spectrogram to line up with the seismogram
+		self.ax[i*self.mult+1].set_xlim(0.25,self.seconds-0.25)
+		self.ax[i*self.mult+1].set_ylim(0,int(self.sps/2))
+		# imshow to update the spectrogram
+		self.ax[i*self.mult+1].imshow(np.flipud(sg**(1/float(10))), cmap='inferno',
+				extent=(self.seconds-(1/(self.sps/float(len(self.stream[i].data)))),
+						self.seconds,0,self.sps/2), aspect='auto')
+		# some things that unfortunately can't be in the setup function:
+		self.ax[i*self.mult+1].tick_params(axis='x', which='both',
+				bottom=False, top=False, labelbottom=False)
+		self.ax[i*self.mult+1].set_ylabel('Frequency (Hz)', color=self.fgcolor)
+		self.ax[i*self.mult+1].set_xlabel('Time (UTC)', color=self.fgcolor)
+
+
 	def update_plot(self):
 		'''
 		Redraw the plot with new data.
@@ -475,56 +586,17 @@ class Plot:
 		self.stream = self.stream.slice(starttime=obstart)	# slice the stream to the specified length (seconds variable)
 		i = 0
 		for i in range(self.num_chans):	# for each channel, update the plots
-			comp = 1/self.per_lap	# spectrogram offset compensation factor
-			r = np.arange(start, end, np.timedelta64(int(1000/self.sps), 'ms'))[-len(
-						self.stream[i].data[int(-self.sps*(self.seconds-(comp/2))):-int(self.sps*(comp/2))]):]
 			mean = int(round(np.mean(self.stream[i].data)))
-			self.lines[i].set_ydata(self.stream[i].data[int(-self.sps*(self.seconds-(comp/2))):-int(self.sps*(comp/2))]-mean)
-			self.lines[i].set_xdata(r)	# (1/self.per_lap)/2
-			self.ax[i*self.mult].set_xlim(left=start.astype(datetime)+timedelta(seconds=comp*1.5),
-										  right=end.astype(datetime))
-			self.ax[i*self.mult].set_ylim(bottom=np.min(self.stream[i].data-mean)
-										  -np.ptp(self.stream[i].data-mean)*0.1,
-										  top=np.max(self.stream[i].data-mean)
-										  +np.ptp(self.stream[i].data-mean)*0.1)
-			if self.deconv:
-				if (self.deconv in 'CHAN'):
-					ch = self.stream[i].stats.channel
-					if ('HZ' in ch) or ('HN' in ch) or ('HE' in ch):
-						unit = rs.UNITS['VEL'][1]
-					elif ('EN' in ch):
-						unit = rs.UNITS['ACC'][1]
-					else:
-						unit = rs.UNITS['CHAN'][1]
-					self.ax[i*self.mult].yaxis.set_major_formatter(EngFormatter(unit='%s' % unit.lower()))
-
+			self._draw_lines(i, start, end, mean)
+			self._set_ch_specific_label(i)
 			if self.spectrogram:
-				self.nfft1 = self._nearest_pow_2(self.sps)	# FFTs run much faster if the number of transforms is a power of 2
-				self.nlap1 = self.nfft1 * self.per_lap
-				if len(self.stream[i].data) < self.nfft1:	# when the number of data points is low, we just need to kind of fake it for a few fractions of a second
-					self.nfft1 = 8
-					self.nlap1 = 6
-				sg = self.ax[i*self.mult+1].specgram(self.stream[i].data-mean,
-							NFFT=self.nfft1, pad_to=int(self.nfft1*4), # previously self.sps*4),
-							Fs=self.sps, noverlap=self.nlap1)[0]	# meat & potatoes
-				self.ax[i*self.mult+1].clear()	# incredibly important, otherwise continues to draw over old images (gets exponentially slower)
-				# cloogy way to shift the spectrogram to line up with the seismogram
-				self.ax[i*self.mult+1].set_xlim(0.25,self.seconds-0.25)
-				self.ax[i*self.mult+1].set_ylim(0,int(self.sps/2))
-				# imshow to update the spectrogram
-				self.ax[i*self.mult+1].imshow(np.flipud(sg**(1/float(10))), cmap='inferno',
-						extent=(self.seconds-(1/(self.sps/float(len(self.stream[i].data)))),
-								self.seconds,0,self.sps/2), aspect='auto')
-				# some things that unfortunately can't be in the setup function:
-				self.ax[i*self.mult+1].tick_params(axis='x', which='both',
-						bottom=False, top=False, labelbottom=False)
-				self.ax[i*self.mult+1].set_ylabel('Frequency (Hz)', color=self.fgcolor)
-				self.ax[i*self.mult+1].set_xlabel('Time (UTC)', color=self.fgcolor)
+				self._update_specgram(i, mean)
 			else:
 				# also can't be in the setup function
 				self.ax[i*self.mult].set_xlabel('Time (UTC)', color=self.fgcolor)
 
-	def loop(self):
+
+	def figloop(self):
 		"""
 		Let some time elapse in order for the plot canvas to draw properly.
 		Must be separate from :py:func:`update_plot()` to avoid a broadcast error early in plotting.
@@ -532,6 +604,49 @@ class Plot:
 		self.fig.canvas.start_event_loop(0.005)
 
 
+	def mainloop(self, i, u):
+		'''
+		The main loop in the :py:func:`rsudp.c_plot.Plot.run`.
+
+		:param int i: number of plot events without clearing the linecache
+		:param int u: queue blocking counter
+		:return: number of plot events without clearing the linecache and queue blocking counter
+		:rtype: int, int
+		'''
+		if i > 10:
+			linecache.clearcache()
+			i = 0
+		else:
+			i += 1
+		self.stream = rs.copy(self.stream)	# essential, otherwise the stream has a memory leak
+		self.raw = rs.copy(self.raw)		# and could eventually crash the machine
+		self.deconvolve()
+		self.update_plot()
+		if u >= 0:				# avoiding a matplotlib broadcast error
+			self.figloop()
+
+		if self.save:
+			# save the plot
+			if (self.save_timer > self.save[0][0]):
+				self._eventsave()
+		u = 0
+		time.sleep(0.005)		# wait a ms to see if another packet will arrive
+		sys.stdout.flush()
+		return i, u
+
+	def qu(self, u):
+		'''
+		Get a queue object and increment the queue counter.
+		This is a way to figure out how many channels have arrived in the queue.
+
+		:param int u: queue blocking counter
+		:return: queue blocking counter
+		:rtype: int
+		'''
+		u += 1 if self.getq() else 0
+		return u
+
+
 	def run(self):
 		"""
 		The heart of the plotting routine.
@@ -552,7 +667,7 @@ class Plot:
 		i = 0	# number of plot events without clearing the linecache
 		u = -1	# number of blocked queue calls (must be -1 at startup)
 		while True: # main loop
-			while True:
+			while True: # sub loop
 				if self.alive == False:	# break if the user has closed the plot
 					break
 				n += 1
@@ -561,31 +676,12 @@ class Plot:
 					self.getq()
 					time.sleep(0.009)		# wait a ms to see if another packet will arrive
 				else:
-					u += 1 if self.getq() else 0
+					u = self.qu(u)
 					if n > (self.delay * rs.numchns):
 						n = 0
 						break
-
 			if self.alive == False:	# break if the user has closed the plot
 				printM('Exiting.', self.sender)
 				break
-
-			if i > 10:
-				linecache.clearcache()
-				i = 0
-			else:
-				i += 1
-			self.stream = rs.copy(self.stream)	# essential, otherwise the stream has a memory leak
-			self.raw = rs.copy(self.raw)		# and could eventually crash the machine
-			self.deconvolve()
-			self.update_plot()
-			if u >= 0:				# avoiding a matplotlib broadcast error
-				self.loop()
-
-			if self.save:
-				if (self.save_timer > self.save[0][0]):
-					self._eventsave()
-			u = 0
-			time.sleep(0.005)		# wait a ms to see if another packet will arrive
-			sys.stdout.flush()
+			i, u = self.mainloop(i, u)
 		return
