@@ -20,6 +20,7 @@ port = 8888				# default listening port
 to = 10					# socket test timeout
 firstaddr = ''			# the first address data is received from
 inv = False				# station inventory
+INVWARN = False			# warning when inventory attachment fails
 region = False
 producer = False 		# flag for producer status
 stn = 'Z0000'			# station name
@@ -39,6 +40,9 @@ UNITS = {'ACC'	: ['Acceleration', 'm/s$^2$'],
 		 'DISP'	: ['Displacement', 'm'],
 		 'CHAN'	: ['channel-specific', ' Counts']}
 
+g = 9.81	# earth gravity in m/s2
+
+
 # get an IP to report to the user
 # from https://stackoverflow.com/questions/166506/finding-local-ip-addresses-using-pythons-stdlib
 def get_ip():
@@ -511,9 +515,9 @@ def get_inventory(sender='get_inventory'):
 		try:
 			printM('Fetching inventory for station %s.%s from Raspberry Shake FDSN.'
 					% (net, stn), sender)
-			
-			inv = read_inventory('https://fdsnws.raspberryshakedata.com/fdsnws/station/1/query?network=%s&station=%s&starttime=%s&level=resp&nodata=404&format=xml'
-								 % (net, stn, str(UTCDateTime.now()-timedelta(seconds=14400))))
+			url = 'https://fdsnws.raspberryshakedata.com/fdsnws/station/1/query?network=%s&station=%s&level=resp&nodata=404&format=xml' % (
+				   net, stn)#, str(UTCDateTime.now()-timedelta(seconds=14400)))
+			inv = read_inventory(url)
 			region = FlinnEngdahl().get_region(inv[0][0].longitude, inv[0][0].latitude)
 			printM('Inventory fetch successful. Station region is %s' % (region), sender)
 		except (IndexError, HTTPError):
@@ -551,7 +555,7 @@ def make_trace(d):
 	:rtype: obspy.core.trace.Trace
 	:return: A fully formed Trace object to build a Stream with
 	'''
-	global producer
+	global INVWARN
 	ch = getCHN(d)						# channel
 	if ch:
 		t = getTIME(d)				# unix epoch time since 1970-01-01 00:00:00Z; "timestamp" in obspy
@@ -565,16 +569,19 @@ def make_trace(d):
 		tr.stats.starttime = UTCDateTime(t, precision=3)
 		if inv:
 			try:
-				tr.attach_response(inv)
-			except:
-				if producer:
-					printE('Could not attach inventory response.')
-					printE('Are you sure you set the station name correctly?', spaces=True)
-					printE('This could indicate a mismatch in the number of data channels', spaces=True)
-					printE('between the inventory and the stream. For example,', spaces=True)
-					printE('if you are receiving RS4D data, please make sure', spaces=True)
-					printE('the inventory you download has 4 channels.', spaces=True)
-				producer = False
+				tr.stats.response = inv.get_response(tr.id, tr.stats.starttime)
+			except Exception as e:
+				if not INVWARN:
+					INVWARN = True
+					printE(e, sender='make_trace')
+					printE('Could not attach inventory response.', sender='make_trace')
+					printE('Are you sure you set the station name correctly?', spaces=True, sender='make_trace')
+					printE('This could indicate a mismatch in the number of data channels', spaces=True, sender='make_trace')
+					printE('between the inventory and the stream. For example,', spaces=True, sender='make_trace')
+					printE('if you are receiving RS4D data, please make sure', spaces=True, sender='make_trace')
+					printE('the inventory you download has 4 channels.', spaces=True, sender='make_trace')
+				else:
+					pass
 		return tr
 
 
@@ -659,243 +666,6 @@ def copy(orig):
 	return stream.copy()
 
 
-def fsec(ti):
-	'''
-	.. versionadded:: 0.4.3
-
-	The Raspberry Shake records at hundredths-of-a-second precision.
-	In order to report time at this precision, we need to do some time-fu.
-
-	This function rounds the microsecond fraction of a
-	:py:class:`obspy.core.utcdatetime.UTCDateTime`
-	depending on its precision, so that it accurately reflects the Raspberry Shake's
-	event measurement precision.
-
-	This is necessary because datetime objects in Python are strange and confusing, and
-	strftime doesn't support fractional returns, only the full integer microsecond field
-	which is an integer right-padded with zeroes. This function uses the ``precision``
-	of a datetime object.
-
-	For example:
-
-	.. code-block:: python
-
-		>>> from obspy import UTCDateTime
-		>>> ti = UTCDateTime(2020, 1, 1, 0, 0, 0, 599000, precision=3)
-		>>> fsec(ti)
-		UTCDateTime(2020, 1, 1, 0, 0, 0, 600000)
-
-	:param ti: time object to convert microseconds for
-	:type ti: obspy.core.utcdatetime.UTCDateTime
-	:return: the hundredth-of-a-second rounded version of the time object passed (precision is 0.01 second)
-	:rtype: obspy.core.utcdatetime.UTCDateTime
-	'''
-	# time in python is weird and confusing, but luckily obspy is better than Python
-	# at dealing with datetimes. all we need to do is tell it what precision we want
-	# and it handles the rounding for us.
-	return UTCDateTime(ti, precision=2)
-
-
-def msg_alarm(event_time):
-	'''
-	This function constructs the ``ALARM`` message as a bytes object.
-	Currently this is only used by :py:class:`rsudp.p_producer.Producer`
-	to construct alarm queue messages.
-
-	For example:
-
-	.. code-block:: python
-
-		>>> from obspy import UTCDateTime
-		>>> ti = UTCDateTime(2020, 1, 1, 0, 0, 0, 599000, precision=3)
-		>>> msg_alarm(ti)
-		b'ALARM 2020-01-01T00:00:00.599Z'
-
-	:param obspy.core.utcdatetime.UTCDateTime event_time: the datetime object to serialize and convert to bytes
-	:rtype: bytes
-	:return: the ``ALARM`` message, ready to be put on the queue
-	'''
-	return b'ALARM %s' % bytes(str(event_time), 'utf-8')
-
-
-def msg_reset(reset_time):
-	'''
-	This function constructs the ``RESET`` message as a bytes object.
-	Currently this is only used by :py:class:`rsudp.p_producer.Producer`
-	to construct reset queue messages.
-
-	For example:
-
-	.. code-block:: python
-
-		>>> from obspy import UTCDateTime
-		>>> ti = UTCDateTime(2020, 1, 1, 0, 0, 0, 599000, precision=3)
-		>>> msg_reset(ti)
-		b'RESET 2020-01-01T00:00:00.599Z'
-
-	:param obspy.core.utcdatetime.UTCDateTime reset_time: the datetime object to serialize and convert to bytes
-	:rtype: bytes
-	:return: the ``RESET`` message, ready to be put on the queue
-	'''
-	return b'RESET %s' % bytes(str(reset_time), 'utf-8')
-
-
-def msg_imgpath(event_time, figname):
-	'''
-	This function constructs the ``IMGPATH`` message as a bytes object.
-	Currently this is only used by :py:class:`rsudp.c_plot.Plot`
-	to construct queue messages containing timestamp and saved image path.
-
-	For example:
-
-	.. code-block:: python
-
-		>>> from obspy import UTCDateTime
-		>>> ti = UTCDateTime(2020, 1, 1, 0, 0, 0, 599000, precision=3)
-		>>> path = '/home/pi/rsudp/screenshots/test.png'
-		>>> msg_imgpath(ti, path)
-		b'IMGPATH 2020-01-01T00:00:00.599Z /home/pi/rsudp/screenshots/test.png'
-
-	:param obspy.core.utcdatetime.UTCDateTime event_time: the datetime object to serialize and convert to bytes
-	:param str figname: the figure path as a string
-	:rtype: bytes
-	:return: the ``IMGPATH`` message, ready to be put on the queue
-	'''
-	return b'IMGPATH %s %s' % (bytes(str(event_time), 'utf-8'), bytes(str(figname), 'utf-8'))
-
-
-def msg_term():
-	'''
-	This function constructs the simple ``TERM`` message as a bytes object.
-
-	.. code-block:: python
-
-		>>> msg_term()
-		b'TERM'
-
-
-	:rtype: bytes
-	:return: the ``TERM`` message
-	'''
-	return b'TERM'
-
-
-def get_msg_time(msg):
-	'''
-	This function gets the time from ``ALARM``, ``RESET``,
-	and ``IMGPATH`` messages as a UTCDateTime object.
-
-	For example:
-
-	.. code-block:: python
-
-		>>> from obspy import UTCDateTime
-		>>> ti = UTCDateTime(2020, 1, 1, 0, 0, 0, 599000, precision=3)
-		>>> path = '/home/pi/rsudp/screenshots/test.png'
-		>>> msg = msg_imgpath(ti, path)
-		>>> msg
-		b'IMGPATH 2020-01-01T00:00:00.599Z /home/pi/rsudp/screenshots/test.png'
-		>>> get_msg_time(msg)
-		UTCDateTime(2020, 1, 1, 0, 0, 0, 599000)
-
-	:param bytes msg: the bytes-formatted queue message to decode
-	:rtype: obspy.core.utcdatetime.UTCDateTime
-	:return: the time embedded in the message
-	'''
-	return UTCDateTime.strptime(msg.decode('utf-8').split(' ')[1], '%Y-%m-%dT%H:%M:%S.%fZ')
-
-
-def get_msg_path(msg):
-	'''
-	This function gets the path from ``IMGPATH`` messages as a string.
-
-	For example:
-
-	.. code-block:: python
-
-		>>> from obspy import UTCDateTime
-		>>> ti = UTCDateTime(2020, 1, 1, 0, 0, 0, 599000, precision=3)
-		>>> path = '/home/pi/rsudp/screenshots/test.png'
-		>>> msg = msg_imgpath(ti, path)
-		>>> msg
-		b'IMGPATH 2020-01-01T00:00:00.599Z /home/pi/rsudp/screenshots/test.png'
-		>>> get_msg_path(msg)
-		'/home/pi/rsudp/screenshots/test.png'
-
-	:param bytes msg: the bytes-formatted queue message to decode
-	:rtype: str
-	:return: the path embedded in the message
-	'''
-	return msg.decode('utf-8').split(' ')[2]
-
-
-def deconvolve(self):
-	'''
-	.. role:: pycode(code)
-		:language: python
-	
-	A central helper function for sub-consumers (i.e. :py:class:`rsudp.c_plot.Plot` or :py:class:`rsudp.c_alert.Alert`)
-	that need to deconvolve their raw data to metric units.
-	Consumers with :py:class:`obspy.core.stream.Stream` objects in :pycode:`self.stream` can use this to deconvolve data
-	if this library's :pycode:`rsudp.raspberryshake.inv` variable
-	contains a valid :py:class:`obspy.core.inventory.inventory.Inventory` object.
-
-	:param self self: The self object of the sub-consumer class calling this function. Must contain :pycode:`self.stream` as a :py:class:`obspy.core.stream.Stream` object.
-	'''
-	self.stream = self.raw.copy()
-	for trace in self.stream:
-		trace.stats.units = self.units
-		g = 9.81	# earth gravity in m/s2
-		output = 'ACC' if self.deconv == 'GRAV' else self.deconv	# if conversion is to gravity
-		if self.deconv:
-			if ('HZ' in trace.stats.channel) or ('HE' in trace.stats.channel) or ('HN' in trace.stats.channel):
-				if self.deconv not in 'CHAN':
-					trace.remove_response(inventory=inv, pre_filt=[0.1, 0.6, 0.95*self.sps, self.sps],
-											output=output, water_level=4.5, taper=False)
-				else:
-					trace.remove_response(inventory=inv, pre_filt=[0.1, 0.6, 0.95*self.sps, self.sps],
-											output='VEL', water_level=4.5, taper=False)
-				if 'ACC' in self.deconv:
-					trace.data = np.gradient(trace.data, 1)
-				elif 'GRAV' in self.deconv:
-					trace.data = np.gradient(trace.data, 1) / g
-					trace.stats.units = 'Earth gravity'
-				elif 'DISP' in self.deconv:
-					trace.data = np.cumsum(trace.data)
-					trace.taper(max_percentage=0.1, side='left', max_length=1)
-					trace.detrend(type='demean')
-				else:
-					trace.stats.units = 'Velocity'
-			elif ('NZ' in trace.stats.channel) or ('NE' in trace.stats.channel) or ('NN' in trace.stats.channel):
-				if self.deconv not in 'CHAN':
-					trace.remove_response(inventory=inv, pre_filt=[0.1, 0.6, 0.95*self.sps, self.sps],
-											output=output, water_level=4.5, taper=False)
-				else:
-					trace.remove_response(inventory=inv, pre_filt=[0.1, 0.6, 0.95*self.sps, self.sps],
-											output='ACC', water_level=4.5, taper=False)
-				if 'VEL' in self.deconv:
-					trace.data = np.cumsum(trace.data)
-					trace.detrend(type='demean')
-				elif 'DISP' in self.deconv:
-					trace.data = np.cumsum(np.cumsum(trace.data))
-					trace.detrend(type='linear')
-				elif 'GRAV' in self.deconv:
-					trace.data = trace.data / g
-					trace.stats.units = 'Earth gravity'
-				else:
-					trace.stats.units = 'Acceleration'
-				if ('ACC' not in self.deconv) and ('CHAN' not in self.deconv):
-					trace.taper(max_percentage=0.1, side='left', max_length=1)
-
-			elif 'HDF' in trace.stats.channel:
-				trace.stats.units = ' counts'	# this is HDF
-
-			else:
-				trace.stats.units = ' counts'	# this is a new one
-
-		else:
-			trace.stats.units = ' counts'		# this is not being deconvolved
-
 class ConsumerThread(Thread):
 	'''
 	The default consumer thread setup.
