@@ -61,6 +61,8 @@ def make_test_settings(settings, inet=False):
 	 ``settings['plot']['units']``            ``'CHAN'``
 	 ``settings['plot']['eq_screenshots']``   ``True``
 	 ``settings['alertsound']['enabled']``    ``True``
+	 ``settings['rsam']['enabled']``          ``True``
+	 ``settings['rsam']['debug']``            ``True``
 	======================================== ===================
 
 	.. note::
@@ -97,6 +99,12 @@ def make_test_settings(settings, inet=False):
 
 	settings['alertsound']['enabled'] = True
 
+	settings['rsam']['enabled'] = True
+	settings['rsam']['debug'] = True
+	settings['rsam']['interval'] = 5
+	settings['rsam']['fwaddr'] = "127.0.0.1"
+	settings['rsam']['fwport'] = 4444
+
 	return settings
 
 
